# -*- coding: utf-8 -*-
###############################################################################
#
#   Module for OpenERP 
#   Copyright (C) 2013 Akretion (http://www.akretion.com).
#   @author Sébastien BEAU <sebastien.beau@akretion.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

"""
Helpers usable in the tests
"""

import mock
from contextlib import contextmanager
from ..unit.backend_adapter import call_to_key


class TestResponder(object):
    """ Used to simulate the calls to Prestashop.

    For a call (request) to Prestashop, returns a stored
    response.
    """

    def __init__(self, responses):
        """
        The responses are stored in dict instances.
        The keys are normalized using the ``call_to_key``
        function which transform the request calls in a
        hashable form.

        :param responses: responses returned by Prestashop
        :type responses: dict
        """
        self._responses = responses

    def search(self, prestashop_model, filters):
        key = call_to_key('search', [prestashop_model, filters])
        assert key in self._responses, (
            "%s not found in prestashop responses" % str(key))
        return self._responses[key]

    def get(self, prestashop_model, id, options=None):
        key = call_to_key('get', [prestashop_model, id, options])
        assert key in self._responses, (
            "%s not found in prestashop responses" % str(key))
        return self._responses[key]


@contextmanager
def mock_api(responses):
    """
    :param responses: responses returned by Prestashop
    :type responses: dict
    """
    get_prestashop_response = TestResponder(responses)
    with mock.patch('prestapyt.PrestaShopWebServiceDict') as API:
        api_mock = mock.MagicMock(name='prestapyt.PrestaShopWebServiceDict')
        API.return_value = api_mock
        api_mock.__enter__.return_value = api_mock
        api_mock.search = get_prestashop_response.search
        api_mock.get = get_prestashop_response.get
        yield
